package com.tzld.piaoquan.growth.common.service.Impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.growth.common.common.base.CommonResponse;
import com.tzld.piaoquan.growth.common.common.enums.MessageAttachmentTypeEnum;
import com.tzld.piaoquan.growth.common.component.HttpPoolClient;
import com.tzld.piaoquan.growth.common.dao.mapper.GuaranteesVideoMapper;
import com.tzld.piaoquan.growth.common.dao.mapper.MessageAttachmentMapper;
import com.tzld.piaoquan.growth.common.model.bo.AdPutFlowParam;
import com.tzld.piaoquan.growth.common.model.bo.VideoDetail;
import com.tzld.piaoquan.growth.common.model.bo.VideoParam;
import com.tzld.piaoquan.growth.common.model.po.*;
import com.tzld.piaoquan.growth.common.model.vo.GuaranteedParam;
import com.tzld.piaoquan.growth.common.service.WeComAccessTokenService;
import com.tzld.piaoquan.growth.common.service.MessageAttachmentService;
import com.tzld.piaoquan.growth.common.utils.DateUtil;
import com.tzld.piaoquan.growth.common.utils.LarkRobotUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.*;
import java.util.concurrent.TimeUnit;

import static com.tzld.piaoquan.growth.common.common.constant.MessageConstant.MAX_VIDEO_NUM;
import static com.tzld.piaoquan.growth.common.common.constant.MessageConstant.appid;
import static com.tzld.piaoquan.growth.common.common.constant.OtherServerURL.POST_ADD_TENCENT;
import static com.tzld.piaoquan.growth.common.common.constant.OtherServerURL.POST_VIDEO_DETAIL_URL;
import static com.tzld.piaoquan.growth.common.common.constant.TimeConstant.MILLISECOND_DAY;
import static com.tzld.piaoquan.growth.common.common.constant.WeComConstant.POST_WE_COM_MEDIA_UPLOAD;


@Slf4j
@Service
public class MessageAttachmentServiceImpl implements MessageAttachmentService {

    @Autowired
    private HttpPoolClient httpPoolClient;

    @Autowired
    private WeComAccessTokenService weComAccessTokenService;

    @Autowired
    private MessageAttachmentMapper messageAttachmentMapper;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private GuaranteesVideoMapper guaranteesVideoMapper;

    @Override
    public List<Long> addMessageAttachment(List<MessageAttachment> messageAttachmentList, Long createTime) {
        List<Long> messageAttachmentIds = new ArrayList<>();
        for (MessageAttachment messageAttachment : messageAttachmentList) {
            MessageAttachmentExample example = new MessageAttachmentExample();
            MessageAttachmentExample.Criteria criteria = example.createCriteria().andTypeEqualTo(messageAttachment.getType());
            if (messageAttachment.getType().equals(MessageAttachmentTypeEnum.IMAGE.getCode())) {
                if (messageAttachment.getMediaId() != null) {
                    criteria.andMediaIdEqualTo(messageAttachment.getMediaId());
                }
                if (messageAttachment.getPicUrl() != null) {
                    criteria.andPicUrlEqualTo(messageAttachment.getPicUrl());
                }
            }
            if (messageAttachment.getType().equals(MessageAttachmentTypeEnum.MINI_PROGRAM.getCode())) {
                if (messageAttachment.getAppid() != null) {
                    criteria.andAppidEqualTo(messageAttachment.getAppid());
                }
                if (messageAttachment.getMiniprogramVideoId() != null) {
                    criteria.andMiniprogramVideoIdEqualTo(messageAttachment.getMiniprogramVideoId());
                }
                if (messageAttachment.getStaffId() != null) {
                    criteria.andStaffIdEqualTo(messageAttachment.getStaffId());
                }
            }

            if (messageAttachment.getType().equals(MessageAttachmentTypeEnum.LINK.getCode())) {
                if (messageAttachment.getTitle() != null) {
                    criteria.andTitleEqualTo(messageAttachment.getTitle());
                }
                if (messageAttachment.getPicUrl() != null) {
                    criteria.andPicUrlEqualTo(messageAttachment.getPicUrl());
                }
                if (messageAttachment.getDesc() != null) {
                    criteria.andDescEqualTo(messageAttachment.getDesc());
                }
                if (messageAttachment.getUrl() != null) {
                    criteria.andUrlEqualTo(messageAttachment.getUrl());
                }
            }

            List<MessageAttachment> messageAttachments = messageAttachmentMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(messageAttachments)) {
                if (messageAttachment.getType().equals(MessageAttachmentTypeEnum.MINI_PROGRAM.getCode()) &&
                        createTime != null) {
                    messageAttachment.setSendTime(new Date(createTime));
                }
                messageAttachmentMapper.insertSelective(messageAttachment);
            } else {
                messageAttachment.setId(messageAttachments.get(0).getId());
                messageAttachmentMapper.updateByPrimaryKeySelective(messageAttachment);
            }
            messageAttachmentIds.add(messageAttachment.getId());
        }
        return messageAttachmentIds;
    }

    @Override
    public List<MessageAttachment> getMessageAttachment(List<Long> attachmentIds) {
        MessageAttachmentExample example = new MessageAttachmentExample();
        example.createCriteria().andIdIn(attachmentIds);
        return messageAttachmentMapper.selectByExample(example);
    }


    public CommonResponse<Void> createGuaranteedMiniProgram(GuaranteedParam guaranteedParam) {
        if (isInvalidGuaranteedParam(guaranteedParam)) {
            return CommonResponse.create(500, "参数错误");
        }
        Set<Long> videoIds = new HashSet<>();
        for (VideoParam videoParam : guaranteedParam.getVideoParamList()) {
            CommonResponse<Void> voidCommonResponse = processVideoParam(videoParam, guaranteedParam.getDate(), videoIds);
            if (voidCommonResponse != null) {
                return voidCommonResponse;
            }
        }

        Map<Long, VideoDetail> coverMap = getVideoDetail(videoIds);
        Map<Long, MessageAttachment> messageAttachmentMap = createMessageAttachmentMap(videoIds, coverMap);

        List<MessageAttachment> messageAttachmentList = new ArrayList<>();
        for (VideoParam videoParam : guaranteedParam.getVideoParamList()) {
            addMessageAttachments(videoParam, messageAttachmentMap, guaranteedParam.getDate(), messageAttachmentList);
        }

        addMessageAttachment(messageAttachmentList, null);
        return CommonResponse.success();
    }

    private boolean isInvalidGuaranteedParam(GuaranteedParam guaranteedParam) {
        return guaranteedParam == null ||
                StringUtils.isEmpty(guaranteedParam.getDate()) ||
                CollectionUtils.isEmpty(guaranteedParam.getVideoParamList());
    }


    private CommonResponse<Void> processVideoParam(VideoParam videoParam, String date, Set<Long> videoIds) {
        if (CollectionUtils.isEmpty(videoParam.getVideoIds()) || videoParam.getVideoIds().size() < MAX_VIDEO_NUM) {
            return CommonResponse.create(500, "保底视频数量异常，请查看" + date);
        }
        List<Long> staffIds = Optional.ofNullable(videoParam.getStaffIds()).orElseGet(() -> {
            List<Long> newStaffIds = new ArrayList<>();
            if (videoParam.getStaffId() != null) {
                newStaffIds.add(videoParam.getStaffId());
            }
            return newStaffIds;
        });
        for (Long staffId : staffIds) {
            CommonResponse<Void> voidCommonResponse = checkVideoSendTime(videoParam, staffId);
            if (voidCommonResponse != null) {
                return voidCommonResponse;
            }
            videoIds.addAll(videoParam.getVideoIds());
        }
        return null;
    }

    private CommonResponse<Void> checkVideoSendTime(VideoParam videoParam, Long staffId) {
        for (Long videoId : videoParam.getVideoIds()) {
            MessageAttachmentExample example = new MessageAttachmentExample();
            example.createCriteria().andMiniprogramVideoIdEqualTo(videoId).andStaffIdEqualTo(staffId);
            List<MessageAttachment> messageAttachmentList = messageAttachmentMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(messageAttachmentList)) {
                continue;
            }
            MessageAttachment messageAttachment = messageAttachmentList.get(0);
            if (messageAttachment.getSendTime() != null &&
                    DateUtil.dateDifference(new Date(), messageAttachment.getSendTime()) < 30 * MILLISECOND_DAY) {
                return CommonResponse.create(500, "保底视频30天内已发送，请查看videoId=" + videoId);
            }
        }
        return null;
    }

    private Map<Long, MessageAttachment> createMessageAttachmentMap(Set<Long> videoIds, Map<Long, VideoDetail> coverMap) {
        Map<Long, MessageAttachment> messageAttachmentMap = new HashMap<>();
        for (Long videoId : videoIds) {
            VideoDetail videoDetail = coverMap.get(videoId);
            if (videoDetail == null || StringUtils.isEmpty(videoDetail.getCover()) || StringUtils.isEmpty(videoDetail.getTitle())) {
                LarkRobotUtil.sendMessage("获取视频详情异常，请查看" + videoId);
                throw new RuntimeException("获取视频详情异常");
            }
            MessageAttachment messageAttachment = new MessageAttachment();
            messageAttachment.setMiniprogramVideoId(videoId);
            messageAttachment.setType(MessageAttachmentTypeEnum.MINI_PROGRAM.getCode());
            messageAttachment.setCover(videoDetail.getCover());
            messageAttachment.setTitle(videoDetail.getTitle());
            messageAttachment.setAppid(appid);
            messageAttachmentMap.put(videoId, messageAttachment);
        }
        return messageAttachmentMap;
    }

    private void addMessageAttachments(VideoParam videoParam, Map<Long, MessageAttachment> messageAttachmentMap, String date, List<MessageAttachment> messageAttachmentList) {
        List<Long> staffIds = Optional.ofNullable(videoParam.getStaffIds()).orElseGet(() -> {
            List<Long> newStaffIds = new ArrayList<>();
            if (videoParam.getStaffId() != null) {
                newStaffIds.add(videoParam.getStaffId());
            }
            return newStaffIds;
        });

        for (Long staffId : staffIds) {
            if (staffId == 0) {
                addGuaranteesVideo(date, staffId, videoParam.getVideoIds());
                continue;
            }

            for (Long videoId : videoParam.getVideoIds()) {
                MessageAttachment messageAttachment = messageAttachmentMap.get(videoId);
                if (messageAttachment == null) {
                    continue;
                }

                MessageAttachment newMessageAttachment = new MessageAttachment();
                BeanUtils.copyProperties(messageAttachment, newMessageAttachment);
                newMessageAttachment.setStaffId(staffId);
                messageAttachmentList.add(newMessageAttachment);
            }
            addGuaranteesVideo(date, staffId, videoParam.getVideoIds());
        }
    }

    private void addGuaranteesVideo(String date, Long staffId, List<Long> videoIds) {
        GuaranteesVideoExample example = new GuaranteesVideoExample();
        example.createCriteria().andIsDeleteEqualTo(0).andDateEqualTo(date).andStaffIdEqualTo(staffId);
        List<GuaranteesVideo> guaranteesVideos = guaranteesVideoMapper.selectByExample(example);
        for (GuaranteesVideo guaranteesVideo : guaranteesVideos) {
            guaranteesVideo.setIsDelete(1);
            guaranteesVideoMapper.updateByPrimaryKeySelective(guaranteesVideo);
        }
        GuaranteesVideo guaranteesVideo = new GuaranteesVideo();
        guaranteesVideo.setDate(date);
        guaranteesVideo.setStaffId(staffId);
        guaranteesVideo.setVideoIds(JSONObject.toJSONString(videoIds));
        guaranteesVideoMapper.insertSelective(guaranteesVideo);
    }

    @Override
    public GuaranteedParam getGuaranteedVideo(String date) {
        GuaranteesVideoExample example = new GuaranteesVideoExample();
        example.createCriteria().andDateEqualTo(date).andIsDeleteEqualTo(0);
        List<GuaranteesVideo> guaranteesVideos = guaranteesVideoMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(guaranteesVideos)) {
            LarkRobotUtil.sendMessage("获保底视频空，@薛一鸣");
            return null;
        }
        GuaranteedParam guaranteedParam = new GuaranteedParam();
        guaranteedParam.setDate(date);
        List<VideoParam> videoParams = new ArrayList<>();
        for (GuaranteesVideo guaranteesVideo : guaranteesVideos) {
            VideoParam videoParam = new VideoParam();
            videoParam.setStaffId(guaranteesVideo.getStaffId());
            List<Long> videoIds = JSONArray.parseArray(guaranteesVideo.getVideoIds(), Long.class);
            videoParam.setVideoIds(videoIds);
            videoParams.add(videoParam);
        }
        guaranteedParam.setVideoParamList(videoParams);
        return guaranteedParam;
    }


    @Override
    public Map<Long, VideoDetail> getVideoDetail(Set<Long> videoIdList) {
        try {
            Map<Long, VideoDetail> map = new HashMap<>();
            JSONObject params = new JSONObject();
            params.put("videoIdList", videoIdList);
            String post = httpPoolClient.post(POST_VIDEO_DETAIL_URL, params.toJSONString());
            JSONObject res = JSONObject.parseObject(post);
            JSONArray data = res.getJSONArray("data");
            for (int i = 0; i < data.size(); i++) {
                VideoDetail videoDetail = new VideoDetail();
                JSONObject jsonObject = data.getJSONObject(i);
                if (Objects.isNull(jsonObject)) {
                    continue;
                }
                Long videoId = jsonObject.getLong("id");
                String shareImgPath = jsonObject.getString("shareImgPath");
                String title = jsonObject.getString("title");
                String videoPath = jsonObject.getString("videoPath");
                videoDetail.setCover(shareImgPath);
                videoDetail.setTitle(title);
                videoDetail.setVideoPath(videoPath);
                map.put(videoId, videoDetail);
            }
            return map;
        } catch (IOException e) {
            log.error("MessageAttachmentServiceImpl getVideDetail error", e);
        }
        return new HashMap<>();
    }

    @Override
    public String getPicMediaId(String cover, Long corpId) {
        String mediaId = (String) redisTemplate.opsForValue().get(cover);
        if (StringUtils.isNotEmpty(mediaId)) {
            return mediaId;
        }
        String filePath = UUID.randomUUID() + ".jpg"; // 临时文件路径
        try {
            HttpURLConnection httpUrl = (HttpURLConnection) new URL(cover).openConnection();
            httpUrl.connect();
            InputStream inputStream = httpUrl.getInputStream();

            // 将文件内容写入临时文件
            try (OutputStream outputStream = Files.newOutputStream(Paths.get(filePath))) {
                byte[] buffer = new byte[4096];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
            }
            inputStream.close();
            httpUrl.disconnect();
            File file = new File(filePath);
            String weComAccessToken = weComAccessTokenService.getWeComAccessToken(corpId);
            String url = String.format(POST_WE_COM_MEDIA_UPLOAD + "?access_token=%s&type=%s", weComAccessToken, "image");
            String res = httpPoolClient.post(url, file);
            JSONObject jsonObject = JSONObject.parseObject(res);
            if (jsonObject != null && jsonObject.getInteger("errcode") == 0) {
                mediaId = jsonObject.getString("media_id");
                redisTemplate.opsForValue().set(cover, mediaId, 2, TimeUnit.DAYS);
            }
            Files.delete(Paths.get(filePath));
        } catch (Exception e) {
            log.error("getPicMediaId error", e);
        }
        return mediaId;
    }


    @Override
    public String getPage(Staff staff, Long videoId) {
        try {
            if (staff == null || staff.getCarrierId() == null) {
                return "";
            }
            String page = selectPage(videoId, staff.getCarrierId(), "企微", "日常推送");
            if (StringUtils.isNotEmpty(page)) {
                return page;
            }
            return insertPage(staff.getCarrierId(), videoId, staff.getRemark());
        } catch (IOException e) {
            log.error("MessageAttachmentService getPage error", e);
        }
        return null;
    }

    private String insertPage(String carrierId, Long videoId, String remark) throws IOException {
        AdPutFlowParam param = new AdPutFlowParam();
        param.setChannel("tencentqw");
        param.setPath("pages/category.html");
        param.setPutCarrierId(carrierId);
        param.setPutScene("touliu");
        param.setPutStartDate(DateUtil.getDayDateString("yyyy-MM-dd"));
        param.setPutTypeOne("企微");
        param.setPutTypeTwo("日常推送");
        param.setPutTypeThree("位置X");
        param.setRemark(remark);
        param.setVideoId(videoId);
        Map<String, String> requestParam = new HashMap<>();
        String jumpPage = "pages/user-videos?id=${videoId}&fromGzh=1&rootShareId=${uuid}&shareId=${uuid}&rootSourceId=[rootSourceId]"
                .replace("${videoId}", "" + videoId)
                .replace("${uuid}", "" + UUID.randomUUID());
        requestParam.put("jumpPage", jumpPage);
        param.setRequestParam(requestParam);
        String res = httpPoolClient.post(POST_ADD_TENCENT, JSONObject.toJSONString(param));
        JSONObject jsonObject = JSONObject.parseObject(res);
        JSONObject data = jsonObject.getJSONObject("data");
        return data.getString("url");
    }

    public String selectPage(Long videoId, String carrierId, String typeOne, String typeTwo) {
        int totalPage = 1;
        for (int n = 1; n <= totalPage; n++) {
            String url = getUrl(videoId, n);
            try {
                String res = httpPoolClient.get(url);
                JSONObject jsonObject = JSONObject.parseObject(res);
                JSONObject data = jsonObject.getJSONObject("data");
                if (data == null) {
                    continue;
                }
                if (data.getInteger("totalPage") != null) {
                    totalPage = data.getInteger("totalPage");
                }
                JSONArray objs = data.getJSONArray("objs");
                for (int i = 0; i < objs.size(); i++) {
                    JSONObject obj = objs.getJSONObject(i);
                    String putCarrierId = obj.getString("putCarrierId");
                    String page = obj.getString("url");
                    String putTypeOne = obj.getString("putTypeOne");
                    String putTypeTwo = obj.getString("putTypeTwo");

                    if (StringUtils.isEmpty(putCarrierId) || StringUtils.isEmpty(page) || StringUtils.isEmpty(putTypeOne) || StringUtils.isEmpty(putTypeTwo)) {
                        continue;
                    }

                    if (putCarrierId.equals(carrierId) && putTypeOne.equals(typeOne) && putTypeTwo.equals(typeTwo)) {
                        return page;
                    }
                }
            } catch (IOException e) {
                log.error("Error fetching data from API for videoId: " + videoId + ", page: " + n, e);
            }
        }
        return null;
    }

    private static String getUrl(Long videoId, int n) {
        String baseUrl = "https://api.piaoquantv.com/ad/put/flow/list/tencent";
        String putScene = "touliu";
        String channel = "tencentqw";
        int pageSize = 20;
        return String.format(
                "%s?videoId=%d&putScene=%s&channel=%s&currentPage=%d&pageSize=%d",
                baseUrl,
                videoId,
                putScene,
                channel,
                n,
                pageSize
        );
    }

    @Override
    public String getPage(String channel, String carrierId, String scene, String putTypeOne,
                          String putTypeTwo, String putTypeThree, Long videoId) {
        AdPutFlowParam param = new AdPutFlowParam();
        param.setChannel(channel);
        param.setPath("pages/category");
        param.setPutCarrierId(carrierId);
        param.setPutScene(scene);
        param.setPutStartDate(DateUtil.getDayDateString("yyyy-MM-dd"));
        param.setPutTypeOne(putTypeOne);
        param.setPutTypeTwo(putTypeTwo);
        param.setPutTypeThree(putTypeThree);
        param.setVideoId(videoId);
        Map<String, String> requestParam = new HashMap<>();
        String jumpPage = "pages/user-videos?id=${videoId}&fromGzh=1&rootShareId=${uuid}&shareId=${uuid}&rootSourceId=[rootSourceId]"
                .replace("${videoId}", "" + videoId)
                .replace("${uuid}", "" + UUID.randomUUID());
        requestParam.put("jumpPage", jumpPage);
        param.setRequestParam(requestParam);
        try {
            String res = httpPoolClient.post(POST_ADD_TENCENT, JSONObject.toJSONString(param));
            JSONObject jsonObject = JSONObject.parseObject(res);
            JSONObject data = jsonObject.getJSONObject("data");
            return data.getString("url");
        } catch (IOException e) {
            log.error("MessageAttachmentService getPage error", e);
        }
        return null;
    }

}
